../../guestbin/libreswan-up-down.sh ikev1-ike=aes128-sha1-dh19 --alive -I 192.0.1.254 192.0.2.254
../../guestbin/libreswan-up-down.sh ikev1-ike=aes128-sha1-dh20 --alive -I 192.0.1.254 192.0.2.254
../../guestbin/libreswan-up-down.sh ikev1-ike=aes128-sha1-dh21 --alive -I 192.0.1.254 192.0.2.254
